# coding: utf-8

"""
    Python SDK for Opsgenie REST API

    Python SDK for Opsgenie REST API  # noqa: E501

    The version of the OpenAPI document: 2.0.0
    Contact: support@opsgenie.com
    Generated by: https://openapi-generator.tech
"""

import six


class OpenApiException(Exception):
    """The base exception class for all OpenAPIExceptions"""


class RetryableException(OpenApiException):
    """The base exception class for all Exceptions for which a retry attempt can be made"""


class ApiTypeError(OpenApiException, TypeError):
    def __init__(self, msg, path_to_item=None, valid_classes=None,
                 key_type=None):
        """ Raises an exception for TypeErrors

        Args:
            msg (str): the exception message

        Keyword Args:
            path_to_item (list): a list of keys an indices to get to the
                                 current_item
                                 None if unset
            valid_classes (tuple): the primitive classes that current item
                                   should be an instance of
                                   None if unset
            key_type (bool): False if our value is a value in a dict
                             True if it is a key in a dict
                             False if our item is an item in a list
                             None if unset
        """
        self.path_to_item = path_to_item
        self.valid_classes = valid_classes
        self.key_type = key_type
        full_msg = msg
        if path_to_item:
            full_msg = "{0} at {1}".format(msg, render_path(path_to_item))
        super(ApiTypeError, self).__init__(full_msg)


class ApiValueError(OpenApiException, ValueError):
    def __init__(self, msg, path_to_item=None):
        """
        Args:
            msg (str): the exception message

        Keyword Args:
            path_to_item (list) the path to the exception in the
                received_data dict. None if unset
        """

        self.path_to_item = path_to_item
        full_msg = msg
        if path_to_item:
            full_msg = "{0} at {1}".format(msg, render_path(path_to_item))
        super(ApiValueError, self).__init__(full_msg)


class ApiKeyError(OpenApiException, KeyError):
    def __init__(self, msg, path_to_item=None):
        """
        Args:
            msg (str): the exception message

        Keyword Args:
            path_to_item (None/list) the path to the exception in the
                received_data dict
        """
        self.path_to_item = path_to_item
        full_msg = msg
        if path_to_item:
            full_msg = "{0} at {1}".format(msg, render_path(path_to_item))
        super(ApiKeyError, self).__init__(full_msg)


class ApiException(RetryableException):

    def __init__(self, status=None, reason=None, http_resp=None):
        if http_resp:
            self.status = http_resp.status
            self.reason = http_resp.reason
            self.body = http_resp.data
            self.headers = http_resp.getheaders()
        else:
            self.status = status
            self.reason = reason
            self.body = None
            self.headers = None

    def __str__(self):
        """Custom error messages for exception"""
        error_message = "({0})\n" \
                        "Reason: {1}\n".format(self.status, self.reason)
        if self.headers:
            error_message += "HTTP response headers: {0}\n".format(
                self.headers)

        if self.body:
            error_message += "HTTP response body: {0}\n".format(self.body)

        return error_message


class AuthenticationException(RetryableException):

    def __init__(self, status=None, reason=None, http_resp=None):
        if http_resp:
            self.status = http_resp.status
            self.reason = http_resp.reason
            self.body = http_resp.data
            self.headers = http_resp.getheaders()
        else:
            self.status = status
            self.reason = reason
            self.body = None
            self.headers = None

    def __str__(self):
        """Custom error messages for exception"""
        error_message = "You are not authorized to perform this action: ({0})\n" \
                        "Reason: {1}\n".format(self.status, self.reason)
        if self.headers:
            error_message += "HTTP response headers: {0}\n".format(
                self.headers)

        if self.body:
            error_message += "HTTP response body: {0}\n".format(self.body)

        return error_message


class ServerErrorException(RetryableException):

    def __init__(self, status=None, reason=None, http_resp=None):
        if http_resp:
            self.status = http_resp.status
            self.reason = http_resp.reason
            self.body = http_resp.data
            self.headers = http_resp.getheaders()
        else:
            self.status = status
            self.reason = reason
            self.body = None
            self.headers = None

    def __str__(self):
        """Custom error messages for exception"""
        error_message = "There is an internal server error: ({0})\n" \
                        "Reason: {1}\n".format(self.status, self.reason)
        if self.headers:
            error_message += "HTTP response headers: {0}\n".format(
                self.headers)

        if self.body:
            error_message += "HTTP response body: {0}\n".format(self.body)

        return error_message


class ConfigurationException(RetryableException):

    def __init__(self, status=None, reason=None, http_resp=None):
        if http_resp:
            self.status = http_resp.status
            self.reason = http_resp.reason
            self.body = http_resp.data
            self.headers = http_resp.getheaders()
        else:
            self.status = status
            self.reason = reason
            self.body = None
            self.headers = None

    def __str__(self):
        """Custom error messages for exception"""
        error_message = "You have configured something incorrectly: ({0})\n" \
                        "Reason: {1}\n".format(self.status, self.reason)
        if self.headers:
            error_message += "HTTP response headers: {0}\n".format(
                self.headers)

        if self.body:
            error_message += "HTTP response body: {0}\n".format(self.body)

        return error_message


def render_path(path_to_item):
    """Returns a string representation of a path"""
    result = ""
    for pth in path_to_item:
        if isinstance(pth, six.integer_types):
            result += "[{0}]".format(pth)
        else:
            result += "['{0}']".format(pth)
    return result


def build_exception(response):
    http_error_code = response.status
    exception = {
        400: ConfigurationException(reason="Invalid JSON body"),
        401: AuthenticationException(reason="apiKey is invalid or integration is disabled"),
        402: AuthenticationException(
            reason="apiKey is valid but the account cannot do this action because of subscription plan"),
        403: AuthenticationException(
            reason="apiKey is valid but the apiKey cannot do this operation because of permissions"),
        404: ApiException(reason="Resource or handler not found"),
        405: AuthenticationException(reason="URL is valid but HTTP method not supported"),
        406: ConfigurationException(reason="Requested format is not supported (Accept header)"),
        409: ConfigurationException(
            reason="ID or name conflicts with another entity. E.g. integration name already exists"),
        410: ConfigurationException(reason="Feature is deprecated"),
        415: ConfigurationException(reason="Request body format is not supported (Content-Type header)"),
        416: ConfigurationException(reason="The given range is not supported."),
        422: ConfigurationException(reason="Semantic errors in request body"),
        428: ConfigurationException(reason="Entity is used by another entity (schedule,escalation,team,etc)"),
        429: ApiException(reason="Throttling"),
        500: ServerErrorException(reason="Internal Server Error"),
        501: ServerErrorException(reason="Not Implemented"),
        502: ServerErrorException(reason="Bad Gateway"),
        503: ServerErrorException(reason="Back-end servers are at capacity"),
        504: ServerErrorException(reason="Gateway Timeout")
    }.get(http_error_code, ApiException(reason="API Encountered a Problem"))

    exception.status = response.status
    exception.reason = response.reason + ": " + exception.reason
    exception.body = response.data
    exception.headers = response.getheaders()

    return exception


def get_all_exceptions():
    return ApiException, AuthenticationException, ServerErrorException, ConfigurationException
